<?php

namespace EventPortal\Domain\Event\Form\Actions;

use EventPortal\Domain\Event\Form\Models\Form;
use EventPortal\Domain\Event\Form\Models\FormStep;
use EventPortal\Domain\Event\Form\Models\FormStepSection;
use EventPortal\Domain\Event\Form\Models\Question;
use EventPortal\Domain\Event\Form\Models\QuestionOption;
use EventPortal\Domain\Event\Form\Models\QuestionValidation;
use EventPortal\Domain\Event\Models\Event;
use Illuminate\Support\Arr;
use Illuminate\Support\Collection;

class CreateForm
{
    public function execute(Event $event, array $form): void
    {
        $steps = collect(Arr::get($form, 'steps'));
        $form = $event->createOrUpdateForm($form);

        $this->deleteUnusedSteps($steps, $form);
        $steps->each(static function ($step) use ($form): void {
            $sections = collect(Arr::get($step, 'sections'));
            $formStep = $form->createOrUpdateStep($step);

            self::deleteUnusedSections($sections, $formStep);
            $sections->each(static function ($section) use ($formStep): void {
                $questions = collect(Arr::get($section, 'questions'));
                $formStepSection = $formStep->createOrUpdateSection($section);

                self::deleteUnusedQuestions($questions, $formStepSection);
                $questions->each(static function ($question) use ($formStepSection): void {
                    $options = collect(Arr::get($question, 'options'));
                    $validations = collect(Arr::get($question, 'validations'));
                    $dbQuestion = $formStepSection->createOrUpdateQuestion($question);

                    self::deleteUnusedValidations($validations, $dbQuestion);
                    if (self::isRequiredValidationNeeded($question, $dbQuestion)) {
                        $validations->prepend([
                            'rule' => 'required',
                            'message' => 'This field is required',
                        ]);
                    }

                    $validations->each(static function ($validation) use ($dbQuestion): void {
                        $dbQuestion->createOrUpdateValidation($validation);
                    });

                    self::deleteUnusedOptions($options, $dbQuestion);

                    $options->each(static function ($option) use ($dbQuestion): void {
                        $dbQuestion->createOrUpdateOption($option);
                    });
                });
            });
        });
    }

    private function deleteUnusedSteps(Collection $steps, Form $form): void
    {
        FormStep::whereNotIn('id', $steps->pluck('id')->filter(null))
            ->where('form_id', $form->id)
            ->delete();
    }

    private static function deleteUnusedSections(Collection $sections, FormStep $formStep): void
    {
        FormStepSection::whereNotIn('id', $sections->pluck('id')->filter(null))
            ->where('form_step_id', $formStep->id)
            ->delete();
    }

    private static function deleteUnusedQuestions(Collection $questions, FormStepSection $formStepSection): void
    {
        Question::whereNotIn('id', $questions->pluck('id')->filter(null))
            ->where('form_step_section_id', $formStepSection->id)
            ->delete();
    }

    private static function deleteUnusedValidations(Collection $validations, Question $question): void
    {
        QuestionValidation::byQuestion($question)
            ->whereNotIn('id', $validations->pluck('id')->filter(null))
            ->delete();
    }

    private static function deleteUnusedOptions(Collection $options, Question $question): void
    {
        QuestionOption::whereNotIn('id', $options->pluck('id')->filter(null))
            ->where('question_id', $question->id)
            ->delete();
    }

    private static function isRequiredValidationNeeded(array $question, Question $dbQuestion): bool
    {
        return Arr::get($question, 'required') === true
            && QuestionValidation::byQuestion($dbQuestion)->isRequired()->exists() === false;
    }
}
