<?php

namespace EventPortal\Domain\Form;

use Closure;
use EventPortal\Domain\Agenda\SessionErrors;
use EventPortal\Domain\Component\Input;
use EventPortal\Domain\Component\Section;
use EventPortal\Domain\Component\Step;
use EventPortal\Domain\Util\RelationshipType;
use EventPortal\Domain\Util\ValidationRules;
use EventPortal\Exceptions\MissingFormStep;
use EventPortal\Exceptions\NoSaveHandlers;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Pipeline\Pipeline;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\ValidationException;

class Form
{
    /** @var string */
    public $name;

    /** @var array */
    public $blocks = [];

    /** @var array */
    public $actions = [];

    /** @var Model */
    private $model;

    /** @var array */
    private $context = [];

    /** @var array */
    private $saveHandlers;

    /** @var bool */
    private $useAlternativeName = false;

    /** @var Validator */
    private $validator;

    /** @var bool */
    public $steps = false;

    public function __construct(string $name)
    {
        $this->name = $name;
    }

    /**
     * @param callable|string $fields
     */
    public function addFields($fields): self
    {
        $this->blocks[] = $fields;

        return $this;
    }

    public function addActions(callable $actions): self
    {
        $this->actions = $actions;

        return $this;
    }

    public function addSteps(callable $steps): self
    {
        $this->blocks[] = $steps;
        $this->steps = true;

        return $this;
    }

    /**
     * @param mixed ...$models
     */
    public function context(...$models): self
    {
        $keyedModels = [];

        foreach ($models as $model) {
            $keyedModels[$model::KEY] = $model;
        }

        $this->context = $keyedModels;

        if ($this->model) {
            $this->context = array_merge($this->context, [$this->model::KEY => $this->model]);
        }

        return $this;
    }

    public function model(Model $model): self
    {
        $this->model = $model;

        if ($this->context) {
            $this->context = array_merge($this->context, [$this->model::KEY => $this->model]);
        }

        return $this;
    }

    public function validate(array $input): self
    {
        $this->resolveWithoutHydration();

        $this->validator = Validator::make($input, ValidationRules::for($this->blocks));

        if ($this->useAlternativeName) {
            return $this->validateUsingAlternativeNames();
        }

        $this->validator->validate();

        return $this;
    }

    public function render(): array
    {
        $this->resolve();

        return [
            'actions' => $this->actions,
            'content' => $this->blocks,
        ];
    }

    public function addSaveHandler(callable $saveHandler): self
    {
        $this->saveHandlers[] = $saveHandler;

        return $this;
    }

    public function useAlternativeNames(): self
    {
        $this->useAlternativeName = true;

        return $this;
    }

    /**
     * @return mixed
     */
    public function save(array $input)
    {
        throw_if($this->saveHandlers === [], NoSaveHandlers::class);

        $payload = Payload::make($input)
            ->withModel($this->model)
            ->withContext($this->context);

        return app(Pipeline::class)
            ->send($payload)
            ->through($this->saveHandlers)
            ->thenReturn();
    }

    private function resolveWithoutHydration(): void
    {
        $this->resolve(false);
    }

    private function resolve(bool $hydrate = true): void
    {
        $this->resolveActions();
        $this->resolveBlocks($hydrate);
    }

    private function resolveActions(): void
    {
        if ($this->actions === []) {
            return;
        }

        $parameters = array_merge($this->context, [$this->model::KEY => $this->model]);

        $this->actions = app()->call($this->actions, $parameters);
    }

    /**
     * Some documentation.
     */
    private function resolveBlocks(bool $hydrate): void
    {
        $this->blocks = collect($this->blocks)
            ->map(function ($block) {
                if ($block instanceof Closure) {
                    $parameters = array_merge($this->context, [$this->model::KEY => $this->model]);

                    return app()->call($block, $parameters);
                }

                return $block;
            })
            ->flatten()
            ->map(function ($step) {
                throw_if($this->steps && ($step instanceof Step) === false, MissingFormStep::class);

                return $this->mapStep($step);
            })
            ->when($hydrate && $this->steps, function (Collection $sections) {
                return $sections->each(function (Step $step) {
                    return $this->hydrate(collect($step->children));
                });
            })
            ->when($hydrate && !$this->steps, function (Collection $sections) {
                return $this->hydrate($sections);
            })
            ->toArray();
    }

    private function hydrate(Collection $sections): Collection
    {
        return $sections->each(function (Section $section): void {
            foreach ($section->flatChildren() as $input) {
                if ($this->useAlternativeName) {
                    $input->useAlternativeName();
                }

                $input->value($this->getValue($input));
            }
        });
    }

    /**
     * @param mixed $step
     * @return Step|Section
     */
    private function mapStep($step)
    {
        if ($step instanceof Step || $step instanceof Section) {
            $step->children = $step->flatChildren();

            return $step;
        }

        return Section::make()->fields([$step]);
    }

    /**
     * @return mixed|null
     */
    private function getValue(Input $input)
    {
        if ($input->value !== null) {
            return $input->value;
        }

        $name = $input->name;

        if ($input->isStatus()) {
            $name = 'readable_status';
        }

        if ($input->isNotScoped()) {
            return optional($this->model)->$name;
        }

        if (RelationshipType::make($this->model, $input->scope)->isBelongsToMany()) {
            $data = $this->model->{$input->scope}->first();

            return $data->pivot->$name;
        }

        return object_get($this->model, $input->scopeKey());
    }

    public function getValidator(): \Illuminate\Validation\Validator
    {
        return $this->validator;
    }

    private function validateUsingAlternativeNames(): self
    {
        try {
            $this->validator->validate();
        } catch (ValidationException $exception) {
            $errors = SessionErrors::swapDatabaseKeysForBrowserKeys($exception->errors());

            foreach ($errors as $key => $value) {
                $this->validator->errors()->add($key, $value);
            }

            throw new ValidationException($this->validator);
        }

        return $this;
    }
}
