<?php

namespace EventPortal\Tests\Unit\Event\Form\Actions;

use EventPortal\Domain\Event\Form\Actions\CreateForm;
use EventPortal\Domain\Event\Form\Enums\FormType;
use EventPortal\Domain\Event\Form\Enums\GdprType;
use EventPortal\Domain\Event\Form\Enums\QuestionType;
use EventPortal\Domain\Event\Form\Models\Form;
use EventPortal\Domain\Event\Form\Models\FormStep;
use EventPortal\Domain\Event\Form\Models\FormStepSection;
use EventPortal\Domain\Event\Form\Models\Question;
use EventPortal\Domain\Event\Form\Models\QuestionOption;
use EventPortal\Domain\Event\Form\Models\QuestionValidation;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;

class UpdateFormTest extends TestCase
{
    use RefreshDatabase;

    /** @test **/
    public function can_update_a_form(): void
    {
        $event = factory(Event::class)->create();
        $form = factory(Form::class)->state('snapshot')->create(['event_id' => $event->id]);
        $step = factory(FormStep::class)->state('snapshot')->create(['form_id' => $form->id]);
        $section = factory(FormStepSection::class)->state('snapshot')->create(['form_step_id' => $step->id]);
        $question = factory(Question::class)->state('snapshot')->create(['form_step_section_id' => $section->id]);
        $otherQuestion = factory(Question::class)->state('snapshot')->create(['form_step_section_id' => $section->id]);
        factory(QuestionOption::class)->state('snapshot')->create(['question_id' => $question->id]);
        factory(QuestionOption::class)->state('snapshot')->create(['question_id' => $question->id]);
        factory(QuestionValidation::class)->state('required')->create(['question_id' => $otherQuestion->id]);

        (new CreateForm)->execute($event, $this->getUpdatedFormData());

        $form = Form::first();
        $this->assertEquals('Updated Test Form', $form->name);
        $step = $form->steps->first();
        $this->assertEquals('Updated Test Stage', $step->name);
        $this->assertEquals('Updated test uuid', $step->uuid);
        $section = $step->sections->first();
        $this->assertEquals('Updated Test Section', $section->name);
        $this->assertEquals('Updated test uuid', $step->uuid);

        $questions = $section->questions->pluck('name');
        $this->assertEquals('Updated test uuid', $section->questions->first()->uuid);
        $this->assertEquals('Updated test uuid', $section->questions->last()->uuid);
        $this->assertEquals('Updated Test Help', $section->questions->first()->help_text);
        $this->assertEquals('after', $section->questions->first()->help_position);
        $this->assertEquals('Updated Test Question', $questions->first());
        $this->assertEquals('New Test Question', $questions->last());
        $this->assertNotContains('Test Question 2', $questions);
        $options = $question->options->pluck('name');
        $this->assertEquals('Updated test uuid', $section->questions->first()->options->first()->uuid);
        $this->assertEquals('Updated test uuid', $section->questions->first()->options->last()->uuid);
        $this->assertEquals('Updated Test Option', $options->first());
        $this->assertEquals('New Test Option', $options->last());
        $this->assertNotContains('Test Option 2', $options);
        $validations = $section->questions->first()->validations;
        $this->assertEquals('Updated test rule', $validations->first()->rule);
        $this->assertEquals('Updated test message', $validations->first()->message);
        $this->assertEquals('required', $validations->last()->rule);
        $this->assertEquals('This field is required', $validations->last()->message);
        $this->assertEmpty($section->questions->last()->validations);
    }

    private function getUpdatedFormData(array $override = []): array
    {
        return array_merge([
            'id' => 1,
            'name' => 'Updated Test Form',
            'type' => FormType::Registration()->getValue(),
            'steps' => [
                [
                    'id' => 1,
                    'name' => 'Updated Test Stage',
                    'order' => 1,
                    'uuid' => 'Updated test uuid',
                    'sections' => [
                        [
                            'id' => 1,
                            'name' => 'Updated Test Section',
                            'order' => 1,
                            'uuid' => 'Updated test uuid',
                            'questions' => [
                                [
                                    'id' => 1,
                                    'name' => 'Updated Test Question',
                                    'type' => QuestionType::Dropdown()->getValue(),
                                    'help_text' => 'Updated Test Help',
                                    'help_position' => 'after',
                                    'gdpr_type' => GdprType::Private()->getValue(),
                                    'order' => 1,
                                    'uuid' => 'Updated test uuid',
                                    'required' => true,
                                    'options' => [
                                        [
                                            'id' => 1,
                                            'name' => 'Updated Test Option',
                                            'order' => 1,
                                            'uuid' => 'Updated test uuid',
                                        ],
                                        [
                                            'name' => 'New Test Option',
                                            'order' => 2,
                                            'uuid' => 'Updated test uuid',
                                        ],
                                    ],
                                    'validations' => [
                                       [
                                           'id' => 1,
                                           'rule' => 'Updated test rule',
                                           'message' => 'Updated test message',
                                       ],
                                    ],
                                ],
                                [
                                    'name' => 'New Test Question',
                                    'type' => QuestionType::SingleLineTextBox()->getValue(),
                                    'help_text' => 'New Test Help',
                                    'help_position' => 'before',
                                    'gdpr_type' => GdprType::Private()->getValue(),
                                    'required' => false,
                                    'order' => 2,
                                    'uuid' => 'Updated test uuid',
                                    'options' => [],
                                    'validations' => [],
                                ],
                            ],
                        ],
                    ],
                ],
            ],
        ], $override);
    }
}
