<?php

namespace EventPortal\Tests\Unit\Event\Form\Requests;

use EventPortal\Domain\Event\Form\Requests\StoreFormRequest;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Support\Facades\Validator;

class StoreFormRequestTest extends TestCase
{
    use RefreshDatabase;

    /** @test **/
    public function form_type_is_required(): void
    {
        $request = (new StoreFormRequest());

        $validator = Validator::make(
            ['type' => ''],
            $request->rules()
        );

        $this->assertEquals('The type field is required.', $validator->errors()->first('type'));
    }

    /** @test **/
    public function form_type_must_be_valid(): void
    {
        $request = (new StoreFormRequest());

        $validator = Validator::make(
            ['type' => 'wrong'],
            $request->rules()
        );

        $this->assertStringContainsString('The type field is not a valid', $validator->errors()->first('type'));
    }

    /** @test **/
    public function steps_is_required(): void
    {
        $request = (new StoreFormRequest());

        $validator = Validator::make(
            ['steps' => []],
            $request->rules()
        );

        $this->assertEquals(
            'The steps field is required.',
            $validator->errors()->first('steps')
        );
    }

    /** @test **/
    public function step_name_is_required(): void
    {
        $request = (new StoreFormRequest());

        $validator = Validator::make(
            ['steps' => [['name' => '']]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.name field is required.',
            $validator->errors()->first('steps.0.name')
        );
    }

    /** @test **/
    public function sections_is_required(): void
    {
        $request = (new StoreFormRequest());

        $validator = Validator::make(
            ['steps' => [['sections' => []]]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.sections field is required.',
            $validator->errors()->first('steps.0.sections')
        );
    }

    /** @test **/
    public function section_name_is_required(): void
    {
        $request = (new StoreFormRequest());

        $validator = Validator::make(
            ['steps' => [['sections' => [['name' => '']]]]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.sections.0.name field is required.',
            $validator->errors()->first('steps.0.sections.0.name')
        );
    }

    /** @test **/
    public function questions_is_required(): void
    {
        $request = (new StoreFormRequest());

        $validator = Validator::make(
            ['steps' => [['sections' => [['questions' => []]]]]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.sections.0.questions field is required.',
            $validator->errors()->first('steps.0.sections.0.questions')
        );
    }

    /** @test **/
    public function question_name_is_required(): void
    {
        $request = (new StoreFormRequest());

        $validator = Validator::make(
            ['steps' => [['sections' => [['questions' => [['name' => '']]]]]]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.sections.0.questions.0.name field is required.',
            $validator->errors()->first('steps.0.sections.0.questions.0.name')
        );
    }

    /** @test **/
    public function question_option_name_and_order_is_required(): void
    {
        $request = (new StoreFormRequest());

        $validator = Validator::make(
            ['steps' => [['sections' => [['questions' => [['options' => [[]]]]]]]]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.sections.0.questions.0.options.0.name field is required.',
            $validator->errors()->first('steps.0.sections.0.questions.0.options.0.name')
        );
        $this->assertEquals(
            'The steps.0.sections.0.questions.0.options.0.order field is required.',
            $validator->errors()->first('steps.0.sections.0.questions.0.options.0.order')
        );
    }

    /** @test **/
    public function question_option_order_must_be_an_integer(): void
    {
        $request = (new StoreFormRequest());

        $validator = Validator::make(
            ['steps' => [['sections' => [['questions' => [['options' => [['order' => 'not-an-int']]]]]]]]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.sections.0.questions.0.options.0.order must be an integer.',
            $validator->errors()->first('steps.0.sections.0.questions.0.options.0.order')
        );
    }

    /** @test **/
    public function question_required_field_is_required(): void
    {
        $request = (new StoreFormRequest());

        $validator = Validator::make(
            ['steps' => [['sections' => [['questions' => [[]]]]]]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.sections.0.questions.0.required field is required.',
            $validator->errors()->first('steps.0.sections.0.questions.0.required')
        );
    }

    /** @test **/
    public function question_required_field_must_be_boolean(): void
    {
        $request = (new StoreFormRequest());

        $validator = Validator::make(
            ['steps' => [['sections' => [['questions' => [['required' => 'NOT BOOLEAN']]]]]]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.sections.0.questions.0.required field must be true or false.',
            $validator->errors()->first('steps.0.sections.0.questions.0.required')
        );
    }

    /** @test **/
    public function question_validation_rule_is_required_with_message(): void
    {
        $request = (new StoreFormRequest());

        $validator = Validator::make(
            ['steps' => [['sections' => [['questions' => [['validations' => [['message' => 'test']]]]]]]]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.sections.0.questions.0.validations.0.rule'.
            ' field is required when steps.0.sections.0.questions.0.validations.0.message is present.',
            $validator->errors()->first('steps.0.sections.0.questions.0.validations.0.rule')
        );
    }

    /** @test **/
    public function question_validation_message_is_required_with_rule(): void
    {
        $request = (new StoreFormRequest());

        $validator = Validator::make(
            ['steps' => [['sections' => [['questions' => [['validations' => [['rule' => 'test']]]]]]]]],
            $request->rules()
        );

        $this->assertEquals(
            'The steps.0.sections.0.questions.0.validations.0.message'.
            ' field is required when steps.0.sections.0.questions.0.validations.0.rule is present.',
            $validator->errors()->first('steps.0.sections.0.questions.0.validations.0.message')
        );
    }
}
